@@ -3,7 +3,7 @@ fitmaxstab <- function(data, coord, cov.mod, loc.form, scale.form, shape.form,
                        temp.form.scale = NULL, temp.form.shape = NULL, iso = FALSE,
                        ..., fit.marge = FALSE, warn = TRUE, method = "Nelder", start,
                        control = list(), weights = NULL, std.err.type = "score",
-                       corr = FALSE){
+                       corr = FALSE, check.grad = FALSE){
 
   if (!(std.err.type) %in% c("none", "score", "grad"))
     stop("'std.err.type' must be one of 'none', 'score' or 'grad'")
@@ -58,25 +58,29 @@ be given for *ALL* GEV parameters")
       control$iter.max <- 10000
   }
 
+  if (check.grad)
+    ## Force to use the nlm optimizer as it returns the numerical gradient
+    method <- "nlm"
+
   if (cov.mod == "gauss")
     fitted <- switch(reg.mod,
                      "full" = smithfull(data, coord, ..., fit.marge = fit.marge, iso = iso,
-                       warn = warn, method = method, control = control,
+                       warn = warn, method = method, control = control, check.grad = check.grad,
                        std.err.type = std.err.type, corr = corr, start = start, weights = weights),
                      "spatgev" = smithform(data, coord, ..., loc.form = loc.form, scale.form = scale.form,
                        shape.form = shape.form, fit.marge = fit.marge, iso = iso, marg.cov = marg.cov,
                        warn = warn, method = method, control = control, std.err.type =
                        std.err.type, corr = corr, start = start, weights = weights,
-                       temp.cov = temp.cov, temp.form.loc = temp.form.loc,
+                       temp.cov = temp.cov, temp.form.loc = temp.form.loc, check.grad = check.grad,
                        temp.form.scale = temp.form.scale, temp.form.shape = temp.form.shape))
 
   else if (cov.mod == "brown")
     fitted <- switch(reg.mod,
                      "full" = brownresnickfull(data, coord, ..., fit.marge = fit.marge,
                        warn = warn, method = method, control = control, std.err.type = std.err.type,
-                       corr = corr, start = start, weights = weights),
+                       corr = corr, start = start, weights = weights, check.grad = check.grad),
                      "spatgev" = brownresnickform(data, coord, ..., loc.form = loc.form,
-                       scale.form = scale.form, shape.form = shape.form,
+                       scale.form = scale.form, shape.form = shape.form, check.grad = check.grad,
                        fit.marge = fit.marge, marg.cov = marg.cov, warn = warn,
                        method = method, control = control, std.err.type = std.err.type, corr = corr,
                        start = start, weights = weights, temp.cov = temp.cov, temp.form.loc = temp.form.loc,
@@ -84,12 +88,12 @@ be given for *ALL* GEV parameters")
   else if (substr(cov.mod, 1, 1) == "i")
       fitted <- switch(reg.mod,
                        "full" = schlatherindfull(data, coord, cov.mod = substr(cov.mod, 2, 8),
-                       ..., fit.marge = fit.marge, warn = warn,
+                       ..., fit.marge = fit.marge, warn = warn, check.grad = check.grad,
                        method = method, control = control, std.err.type = std.err.type,
                        corr = corr, start = start, weights = weights),
                        "spatgev" = schlatherindform(data, coord, cov.mod = substr(cov.mod, 2, 8), ...,
                        loc.form = loc.form, scale.form = scale.form, shape.form = shape.form,
-                       fit.marge = fit.marge, marg.cov = marg.cov, warn = warn,
+                       fit.marge = fit.marge, marg.cov = marg.cov, warn = warn, check.grad = check.grad,
                        method = method, control = control, std.err.type = std.err.type, corr = corr,
                        start = start, weights = weights, temp.cov = temp.cov, temp.form.loc = temp.form.loc,
                        temp.form.scale = temp.form.scale, temp.form.shape = temp.form.shape))
@@ -97,12 +101,12 @@ be given for *ALL* GEV parameters")
   else if (substr(cov.mod, 1, 1) == "g")
       fitted <- switch(reg.mod,
                        "full" = geomgaussfull(data, coord, cov.mod = substr(cov.mod, 2, 8),
-                       ..., fit.marge = fit.marge, warn = warn,
+                       ..., fit.marge = fit.marge, warn = warn, check.grad = check.grad,
                        method = method, control = control, std.err.type = std.err.type,
                        corr = corr, start = start, weights = weights),
                        "spatgev" = geomgaussform(data, coord, cov.mod = substr(cov.mod, 2, 8), ...,
                        loc.form = loc.form, scale.form = scale.form, shape.form = shape.form,
-                       fit.marge = fit.marge, marg.cov = marg.cov, warn = warn,
+                       fit.marge = fit.marge, marg.cov = marg.cov, warn = warn, check.grad = check.grad,
                        method = method, control = control, std.err.type = std.err.type, corr = corr,
                        start = start, weights = weights, temp.cov = temp.cov, temp.form.loc = temp.form.loc,
                        temp.form.scale = temp.form.scale, temp.form.shape = temp.form.shape))
@@ -110,12 +114,12 @@ be given for *ALL* GEV parameters")
   else if (substr(cov.mod, 1, 1) == "t")
       fitted <- switch(reg.mod,
                        "full" = extremaltfull(data, coord, cov.mod = substr(cov.mod, 2, 8),
-                       ..., fit.marge = fit.marge, warn = warn,
+                       ..., fit.marge = fit.marge, warn = warn, check.grad = check.grad,
                        method = method, control = control, std.err.type = std.err.type,
                        corr = corr, start = start, weights = weights),
                        "spatgev" = extremaltform(data, coord, cov.mod = substr(cov.mod, 2, 8), ...,
                        loc.form = loc.form, scale.form = scale.form, shape.form = shape.form,
-                       fit.marge = fit.marge, marg.cov = marg.cov, warn = warn,
+                       fit.marge = fit.marge, marg.cov = marg.cov, warn = warn, check.grad = check.grad,
                        method = method, control = control, std.err.type = std.err.type, corr = corr,
                        start = start, weights = weights, temp.cov = temp.cov, temp.form.loc = temp.form.loc,
                        temp.form.scale = temp.form.scale, temp.form.shape = temp.form.shape))
@@ -123,12 +127,12 @@ be given for *ALL* GEV parameters")
   else
       fitted <- switch(reg.mod,
                        "full" = schlatherfull(data, coord, cov.mod = cov.mod,
-                       ..., fit.marge = fit.marge, warn = warn,
+                       ..., fit.marge = fit.marge, warn = warn, check.grad = check.grad,
                        method = method, control = control, std.err.type = std.err.type,
                        corr = corr, start = start, weights = weights),
                        "spatgev" = schlatherform(data, coord, cov.mod = cov.mod, ...,
                        loc.form = loc.form, scale.form = scale.form, shape.form = shape.form,
-                       fit.marge = fit.marge, marg.cov = marg.cov, warn = warn,
+                       fit.marge = fit.marge, marg.cov = marg.cov, warn = warn, check.grad = check.grad,
                        method = method, control = control, std.err.type = std.err.type, corr = corr,
                        start = start, weights = weights, temp.cov = temp.cov, temp.form.loc = temp.form.loc,
                        temp.form.scale = temp.form.scale, temp.form.shape = temp.form.shape))
