@@ -117,6 +117,7 @@ predict.maxstab <- function(object, newdata, ret.per = NULL,
     ans <- cbind(ans, loc.std.err = loc.std.err, scale.std.err = scale.std.err,
                  shape.std.err = shape.std.err)
 
+  
   if (!is.null(ret.per)){
     ret.lev <- NULL
     for (T in ret.per)
@@ -124,9 +125,36 @@ predict.maxstab <- function(object, newdata, ret.per = NULL,
                                       ans[,"scale"],
                                       ans[,"shape"]))
 
-  colnames(ret.lev) <- paste("Q", ret.per, sep="")
-  ans <- cbind(ans, ret.lev)
-  }
+    colnames(ret.lev) <- paste("Q", ret.per, sep="")
+    ans <- cbind(ans, ret.lev)
+
+    if (std.err && is.matrix(object$var.cov)){
+      par.est <- object$fitted
+      idx.loc <- which(substr(names(par.est), 1, 8) == "locCoeff")
+      idx.scale <- which(substr(names(par.est), 1, 10) == "scaleCoeff")
+      idx.shape <- which(substr(names(par.est), 1, 10) == "shapeCoeff")
+      idx.all <- c(idx.loc, idx.scale, idx.shape)
+
+      ret.lev.std.err <- NULL
+      for (i in 1:length(ret.per)){
+        gradient <- cbind(loc.dsgnmat, (ret.lev[,i] - ans[,"loc"]) / ans[,"scale"] *
+                          scale.dsgnmat,
+                          (-log(-log(1 - 1 /ret.per[i])) * ans[,"scale"] / ans[,"shape"] *
+                         (- log(1 - 1 / ret.per[i]))^(-ans[,"shape"]) -
+                         (ret.lev[,i] - ans[,"loc"]) / ans[,"shape"]) * shape.dsgnmat)
+
+      ret.lev.std.err <- cbind(ret.lev.std.err,
+                               sqrt(diag(gradient %*% object$var.cov[idx.all, idx.all] %*%
+                                         t(gradient))))
+      }
+
+      colnames(ret.lev.std.err) <- paste("Q", ret.per, "std.err", sep ="")
+      ans <- cbind(ans, ret.lev.std.err)
+    }
+    
+    else
+      ret.lev.std.err <- NULL
+    }
   
   return(ans)
 }
