@@ -2,7 +2,7 @@
 
 void rextremalttbm(double *coord, int *nObs, int *nSite, int *dim,
 		   int *covmod, int *grid, double *nugget, double *range,
-		   double *smooth, double *DoF, int *blockSize, int *nlines,
+		   double *smooth, double *DoF, double *uBound, int *nlines,
 		   double *ans){
   /* This function generates random fields from the Extremal-t model
 
@@ -22,14 +22,13 @@ void rextremalttbm(double *coord, int *nObs, int *nSite, int *dim,
 
   int i, neffSite, lagi = 1, lagj = 1;
   double sill = 1 - *nugget;
+  const double irange = 1 / *range;
 
   //rescale the coordinates
-  for (i=(*nSite * *dim);i--;){
-    const double irange = 1 / *range;
-    coord[i] = coord[i] * irange;
-  }
+  for (i=(*nSite * *dim);i--;)
+        coord[i] = coord[i] * irange;
 
-  double *lines = (double *)R_alloc(3 * *nlines, sizeof(double));
+  double *lines = malloc(3 * *nlines * sizeof(double));
   
   if ((*covmod == 3) && (*smooth == 2))
     //This is the gaussian case
@@ -48,15 +47,15 @@ void rextremalttbm(double *coord, int *nObs, int *nSite, int *dim,
     lagj = *nObs;
   }
 
-  double *gp = (double *)R_alloc(neffSite, sizeof(double));
+  double *gp = malloc(neffSite * sizeof(double));
 
   GetRNGstate();
   
   for (i=*nObs;i--;){
-    int l;
-    for (l=*blockSize;l--;){
-      int j;	
-      double scaleStudent = sqrt(*DoF / rchisq(*DoF));
+    int nKO = neffSite;
+    double poisson = 0;
+
+    while (nKO){
 
       /* ------- Random rotation of the lines ----------*/
       double u = unif_rand() - 0.5,
@@ -72,32 +71,43 @@ void rextremalttbm(double *coord, int *nObs, int *nSite, int *dim,
       rotation(lines, nlines, &u, &v, &w, &angle);
       /* -------------- end of rotation ---------------*/
       
+      poisson += exp_rand();
+      double ipoisson = 1 / poisson,
+	thresh = *uBound * ipoisson;
+      
       /* We simulate one realisation of a gaussian random field with
 	 the required covariance function */
-      for (j=neffSite;j--;)
+      for (int j=neffSite;j--;)
 	gp[j] = 0;
+
       tbmcore(nSite, &neffSite, dim, covmod, grid, coord, nugget,
 	      &sill, range, smooth, nlines, lines, gp);
       
-      for (j=neffSite;j--;)
-	ans[j * lagj + i * lagi] = fmax2(gp[j] * scaleStudent, ans[j * lagj + i * lagi]);
-      
+      nKO = neffSite;
+      for (int j=neffSite;j--;){
+	double dummy = R_pow(fmax2(0, gp[j]), *DoF) * ipoisson;
+	ans[j * lagj + i * lagi] = fmax2(dummy, ans[j * lagj + i * lagi]);
+	nKO -= (thresh <= ans[j * lagj + i * lagi]);
+      }    
     }
   }
-
+  
   PutRNGstate();
-
-  //Lastly we multiply by the normalizing constant i.e. (M_k - b_k) / a_k
-  double ia_k = 1 / qt(1 - 1 / (double) *blockSize, *DoF, 1, 0);
+  
+  //Lastly we multiply by the normalizing constant
+  const double imean = M_SQRT_PI * R_pow(2, -0.5 * (*DoF - 2)) /
+    gammafn(0.5 * (*DoF + 1));
+  
   for (i=(neffSite * *nObs);i--;)
-    ans[i] = R_pow(ans[i] * ia_k, *DoF);
-
+    ans[i] *= imean;
+  
+  free(lines); free(gp);
   return;
 }
 
 void rextremaltdirect(double *coord, int *nObs, int *nSite, int *dim,
 		      int *covmod, int *grid, double *nugget, double *range,
-		      double *smooth, double *DoF, int *blockSize, double *ans){
+		      double *smooth, double *DoF, double *uBound, double *ans){
   /* This function generates random fields for the Extremal-t model
 
      coord: the coordinates of the locations
@@ -113,8 +123,8 @@ void rextremaltdirect(double *coord, int *nObs, int *nSite, int *dim,
  blockSize: see rextremalttbm.
        ans: the generated random field */
 
-  int i, j, k, lwork, info = 0, neffSite, lagi = 1, lagj = 1;
-  double sill = 1 - *nugget, one = 1, zero = 0, *work, tmp, sum, dummy;
+  int neffSite, lagi = 1, lagj = 1, oneInt = 1;
+  double sill = 1 - *nugget, one = 1, zero = 0, dummy;
 
   if (*grid){
     neffSite = R_pow_di(*nSite, *dim);
@@ -126,94 +136,62 @@ void rextremaltdirect(double *coord, int *nObs, int *nSite, int *dim,
     lagj = *nObs;
   }
 
-  double *covmat = (double *)R_alloc(neffSite * neffSite, sizeof(double)),
-    *d = (double *)R_alloc(neffSite, sizeof(double)),
-    *u = (double *)R_alloc(neffSite * neffSite, sizeof(double)),
-    *v = (double *)R_alloc(neffSite * neffSite, sizeof(double)),
-    *xvals = (double *) R_alloc(neffSite * neffSite, sizeof(double)),
-    *gp = (double *)R_alloc(neffSite, sizeof(double));
+  double *covmat = malloc(neffSite * neffSite * sizeof(double)),
+    *gp = malloc(neffSite * sizeof(double));
 
   buildcovmat(nSite, grid, covmod, coord, dim, nugget, &sill, range,
 	      smooth, covmat);
   
-  /* Compute the singular value decomposition of the covariance
-     matrix.
+  /* Compute the Cholesky decomposition of the covariance matrix */
+  int info = 0;
+  F77_CALL(dpotrf)("U", &neffSite, covmat, &neffSite, &info);
 
-     This piece of code is strongly inspired from Lapack.c */
-  
-  Memcpy(xvals, covmat, neffSite * neffSite);
-  
-  {
-    int *iwork= (int *) R_alloc(8 * neffSite, sizeof(int));
-    
-    /* ask for optimal size of work array */
-    lwork = -1;
-    F77_CALL(dgesdd)("A", &neffSite, &neffSite, xvals, &neffSite, d, u,
-		     &neffSite, v, &neffSite, &tmp, &lwork, iwork, &info);
-    if (info != 0)
-      error("error code %d from Lapack routine '%s'", info, "dgesdd");
-
-    lwork = (int) tmp;
-    work = (double *) R_alloc(lwork, sizeof(double));
-
-    F77_CALL(dgesdd)("A", &neffSite, &neffSite, xvals, &neffSite, d, u,
-		     &neffSite, v, &neffSite, work, &lwork, iwork, &info);
-    if (info != 0)
-      error("error code %d from Lapack routine '%s'", info, "dgesdd");
-  }
-
-  /*--------------- end of singular value decomposition ---------------*/
-
-  /* Compute the square root of the covariance matrix */
-  // a) First compute diag(sqrt(d)) %*% u
-  for (i=neffSite;i--;){
-    dummy = sqrt(d[i]);
-    
-    for (j=neffSite;j--;)
-      u[i + neffSite * j] *= dummy;
-  }
-
-  // b) Then compute v^T %*% diag(sqrt(d)) %*% u and put it in covmat
-  F77_CALL(dgemm)("T", "N", &neffSite, &neffSite, &neffSite, &one,
-		  v, &neffSite, u, &neffSite, &zero, covmat, &neffSite);
+  if (info != 0)
+    error("error code %d from Lapack routine '%s'", info, "dpotrf");
   
   GetRNGstate();
  
-  for (i=*nObs;i--;){
-    int l;
-    for (l=*blockSize;l--;) {
-      double scaleStudent = sqrt(*DoF / rchisq(*DoF));
+  for (int i=*nObs;i--;){
+    double poisson = 0;
+    int nKO = neffSite;
+      
+    while (nKO){
+      poisson += exp_rand();
+
+      double ipoisson = 1 / poisson,
+	thresh = *uBound * ipoisson;
+
+      
       /* We simulate one realisation of a gaussian random field with
 	 the required covariance function */
-      for (j=neffSite;j--;)
-	d[j] = norm_rand();
+      for (int j=neffSite;j--;)
+	gp[j] = norm_rand();
       
-      for (j=neffSite;j--;){
-	sum = 0;
-	for (k=neffSite;k--;)
-	  sum += d[k] * covmat[j + k * neffSite];
-	
-	gp[j] = sum;
+      F77_CALL(dtrmv)("U", "T", "N", &neffSite, covmat, &neffSite, gp, &oneInt);
+      
+      nKO = neffSite;
+      for (int j=neffSite;j--;){
+	double dummy = R_pow(fmax2(0, gp[j]), *DoF) * ipoisson;
+	ans[j * lagj + i * lagi] = fmax2(dummy, ans[j * lagj + i * lagi]);
+	nKO -= (thresh <= ans[j * lagj + i * lagi]);
       }
-	
-      for (j=neffSite;j--;)
-	ans[j * lagj + i * lagi] = fmax2(gp[j] * scaleStudent, ans[j * lagj + i * lagi]);
-      
     }
   }
 
   PutRNGstate();
-  //Lastly we multiply by the normalizing constant i.e. (M_k - b_k) / a_k
-  double ia_k = 1 / qt(1 - 1 / (double) *blockSize, *DoF, 1, 0);
-  for (i=(neffSite * *nObs);i--;)
-    ans[i] = R_pow(ans[i] * ia_k, *DoF);
+  //Lastly we multiply by the normalizing constant
+  const double imean = M_SQRT_PI * R_pow(2, -0.5 * (*DoF - 2)) /
+    gammafn(0.5 * (*DoF + 1));
+  for (int i=(neffSite * *nObs);i--;)
+    ans[i] *= imean;
   
+  free(covmat); free(gp);
   return;
 }
 
 void rextremaltcirc(int *nObs, int *ngrid, double *steps, int *dim,
 		    int *covmod, double *nugget, double *range,
-		    double *smooth, double *DoF, int *blockSize, double *ans){
+		    double *smooth, double *DoF, double *uBound, double *ans){
   /* This function generates random fields from the Schlather model
 
      nObs: the number of observations to be generated
@@ -250,8 +228,7 @@ blockSize: see rextremalttbm
   /* ---------- beginning of the embedding stage ---------- */
   int mbar = m * m, halfM = m / 2, notPosDef = 0;
   do {
-    double *dist;
-    dist = (double *)R_alloc(mbar, sizeof(double));
+    double *dist = (double *)R_alloc(mbar, sizeof(double));
 
     notPosDef = 0;
     //Computation of the distance
@@ -335,34 +312,41 @@ blockSize: see rextremalttbm
   int mdag = m / 2 + 1, mdagbar = mdag * mdag;
   double isqrtMbar = 1 / sqrt(mbar);
 
-  double *a = (double *)R_alloc(mbar, sizeof(double)),
-    *ia = (double *)R_alloc(mbar, sizeof(double)),
-    *gp = (double *)R_alloc(nbar, sizeof(double));
+  double *a = malloc(mbar * sizeof(double)),
+    *ia = malloc(mbar * sizeof(double)),
+    *gp = malloc(nbar * sizeof(double));
 
   GetRNGstate();
-  for (i=*nObs;i--;){
-    int l;
-    
-    for (l=*blockSize;l--;) {
-      int j;
-      double scaleStudent = sqrt(*DoF / rchisq(*DoF));
+  for (int i=*nObs;i--;){
+    int nKO = nbar;
+    double poisson = 0;
+
+    while (nKO){
+      poisson += exp_rand();
+      double ipoisson = 1 / poisson,
+	thresh = *uBound * ipoisson;
       
       /* We simulate one realisation of a gaussian random field with
 	 the required covariance function */
       circcore(rho, a, ia, m, halfM, mdag, mdagbar, *ngrid, nbar, isqrtMbar, *nugget, gp);
       
-      for (j=nbar;j--;)
-	ans[j + i * nbar] = fmax2(gp[j] * scaleStudent, ans[j + i * nbar]);
-
+      nKO = nbar;
+      for (int j=nbar;j--;){
+	double dummy = R_pow(fmax2(gp[j], 0), *DoF) * ipoisson;
+	ans[j + i * nbar] = fmax2(dummy, ans[j + i * nbar]);
+	nKO -= (thresh <= ans[j + i * nbar]);
+      }
     }
   }
   
   PutRNGstate();
   
-  //Lastly we multiply by the normalizing constant i.e. (M_k - b_k) / a_k
-  double ia_k = 1 / qt(1 - 1 / (double) *blockSize, *DoF, 1, 0);
+  //Lastly we multiply by the normalizing constant
+  const double imean = M_SQRT_PI * R_pow(2, -0.5 * (*DoF - 2)) /
+    gammafn(0.5 * (*DoF + 1));
   for (i=(nbar * *nObs);i--;)
-    ans[i] = R_pow(ans[i] * ia_k, *DoF);
+    ans[i] *= imean;
   
+  free(a); free(ia); free(gp);
   return;
 }
