@@ -21,17 +21,15 @@ void rschlathertbm(double *coord, int *nObs, int *nSite, int *dim,
 
   /* lagi, lagj are integers useful to fill in the output depending on
      if locations are on a grid or not */
-  int i, neffSite, lagi = 1, lagj = 1;
-  const double zero = 0;
+  int neffSite, lagi = 1, lagj = 1;
   double sill = 1 - *nugget;
-  
-  //rescale the coordinates
-  for (i=(*nSite * *dim);i--;){
-    const double irange = 1 / *range;
-    coord[i] = coord[i] * irange;
-  }
+  const double irange = 1 / *range;
 
-  double *lines = (double *)R_alloc(3 * *nlines, sizeof(double));
+  //rescale the coordinates
+  for (int i= 0; i<(*nSite * *dim);i++)    
+    coord[i] = coord[i] * irange;
+
+  double *lines = malloc(3 * *nlines * sizeof(double));
   
   if ((*covmod == 3) && (*smooth == 2))
     //This is the gaussian case
@@ -52,16 +50,16 @@ void rschlathertbm(double *coord, int *nObs, int *nSite, int *dim,
 
   GetRNGstate();
 
-  for (i=*nObs;i--;){
+  double *gp = malloc(neffSite * sizeof(double));
+
+  for (int i = 0; i < *nObs;i++){
     int nKO = neffSite;
     double poisson = 0;
     
     while (nKO) {
       /* The stopping rule is reached when nKO = 0 i.e. when each site
 	 satisfies the condition in Eq. (8) of Schlather (2002) */
-      int j;
-      double *gp = (double *)R_alloc(neffSite, sizeof(double));
-      
+            
       /* ------- Random rotation of the lines ----------*/
       double u = unif_rand() - 0.5,
 	v = unif_rand() - 0.5,
@@ -82,13 +80,13 @@ void rschlathertbm(double *coord, int *nObs, int *nSite, int *dim,
       
       /* We simulate one realisation of a gaussian random field with
 	 the required covariance function */
-      for (j=neffSite;j--;)
+      for (int j = 0; j < neffSite; j++)
 	gp[j] = 0;
       tbmcore(nSite, &neffSite, dim, covmod, grid, coord, nugget,
 	      &sill, range, smooth, nlines, lines, gp);
       
       nKO = neffSite;
-      for (j=neffSite;j--;){
+      for (int j = 0; j < neffSite; j++){
 	ans[j * lagj + i * lagi] = fmax2(gp[j] * ipoisson, ans[j * lagj + i * lagi]);
 	nKO -= (thresh <= ans[j * lagj + i * lagi]);
 	
@@ -100,9 +98,10 @@ void rschlathertbm(double *coord, int *nObs, int *nSite, int *dim,
 
   //Lastly we multiply by 1 / E[max(0, Y)]
   const double imean = M_SQRT2 * M_SQRT_PI;
-  for (i=(neffSite * *nObs);i--;)    
+  for (int i=0; i < (neffSite * *nObs); i++)    
     ans[i] *= imean;
 
+  free(lines); free(gp);
   return;
 }
 
@@ -122,9 +121,8 @@ void rschlatherdirect(double *coord, int *nObs, int *nSite, int *dim,
     smooth: the smooth parameter
        ans: the generated random field */
 
-  int i, j, k, lwork, nKO, info = 0, neffSite, lagi = 1, lagj = 1;
-  double poisson, ipoisson, thresh, sill = 1 - *nugget,
-    one = 1, zero = 0, *work, tmp, sum, dummy;
+  int neffSite, lagi = 1, lagj = 1, oneInt = 1;
+  double sill = 1 - *nugget, one = 1, zero = 0;
 
   if (*grid){
     neffSite = R_pow_di(*nSite, *dim);
@@ -136,85 +134,40 @@ void rschlatherdirect(double *coord, int *nObs, int *nSite, int *dim,
     lagj = *nObs;
   }
 
-  double *covmat = (double *)R_alloc(neffSite * neffSite, sizeof(double)),
-    *d = (double *)R_alloc(neffSite, sizeof(double)),
-    *u = (double *)R_alloc(neffSite * neffSite, sizeof(double)),
-    *v = (double *)R_alloc(neffSite * neffSite, sizeof(double)),
-    *xvals = (double *) R_alloc(neffSite * neffSite, sizeof(double)),
-    *gp = (double *)R_alloc(neffSite, sizeof(double));
+  double *covmat = malloc(neffSite * neffSite * sizeof(double)),
+    *gp = malloc(neffSite * sizeof(double));
 
   buildcovmat(nSite, grid, covmod, coord, dim, nugget, &sill, range,
 	      smooth, covmat);
   
-  /* Compute the singular value decomposition of the covariance
-     matrix.
+  /* Compute the Cholesky decomposition of the covariance matrix */
+  int info = 0;
+  F77_CALL(dpotrf)("U", &neffSite, covmat, &neffSite, &info);
 
-     This piece of code is strongly inspired from Lapack.c */
-  
-  Memcpy(xvals, covmat, neffSite * neffSite);
-  
-  {
-    int *iwork= (int *) R_alloc(8 * neffSite, sizeof(int));
-    
-    /* ask for optimal size of work array */
-    lwork = -1;
-    F77_CALL(dgesdd)("A", &neffSite, &neffSite, xvals, &neffSite, d, u,
-		     &neffSite, v, &neffSite, &tmp, &lwork, iwork, &info);
-    if (info != 0)
-      error("error code %d from Lapack routine '%s'", info, "dgesdd");
+  if (info != 0)
+    error("error code %d from Lapack routine '%s'", info, "dpotrf");
 
-    lwork = (int) tmp;
-    work = (double *) R_alloc(lwork, sizeof(double));
-
-    F77_CALL(dgesdd)("A", &neffSite, &neffSite, xvals, &neffSite, d, u,
-		     &neffSite, v, &neffSite, work, &lwork, iwork, &info);
-    if (info != 0)
-      error("error code %d from Lapack routine '%s'", info, "dgesdd");
-  }
-
-  /*--------------- end of singular value decomposition ---------------*/
-
-  /* Compute the square root of the covariance matrix */
-  // a) First compute diag(sqrt(d)) %*% u
-  for (i=neffSite;i--;){
-    dummy = sqrt(d[i]);
-    
-    for (j=neffSite;j--;)
-      u[i + neffSite * j] *= dummy;
-  }
-
-  // b) Then compute v^T %*% diag(sqrt(d)) %*% u and put it in covmat
-  F77_CALL(dgemm)("T", "N", &neffSite, &neffSite, &neffSite, &one,
-		  v, &neffSite, u, &neffSite, &zero, covmat, &neffSite);
-  
   GetRNGstate();
  
-  for (i=*nObs;i--;){
-    poisson = 0;
-    nKO = neffSite;
-    
+  for (int i = 0; i < *nObs; i++){
+    double poisson = 0;
+    int nKO = neffSite;
+        
     while (nKO) {
       /* The stopping rule is reached when nKO = 0 i.e. when each site
 	 satisfies the condition in Eq. (8) of Schlather (2002) */
       poisson += exp_rand();
-      ipoisson = 1 / poisson;
-      thresh = *uBound * ipoisson;
-      
+      double ipoisson = 1 / poisson, thresh = *uBound * ipoisson;
+            
       /* We simulate one realisation of a gaussian random field with
 	 the required covariance function */
-      for (j=neffSite;j--;)
-	d[j] = norm_rand();
-      
-      for (j=neffSite;j--;){
-	sum = 0;
-	for (k=neffSite;k--;)
-	  sum += d[k] * covmat[j + k * neffSite];
-	
-	gp[j] = sum;
-      }
+      for (int j=0;j<neffSite;j++)
+	gp[j] = norm_rand();
       
+      F77_CALL(dtrmv)("U", "T", "N", &neffSite, covmat, &neffSite, gp, &oneInt);
+
       nKO = neffSite;
-      for (j=neffSite;j--;){
+      for (int j = 0; j < neffSite; j++){
 	ans[j * lagj + i * lagi] = fmax2(gp[j] * ipoisson, ans[j * lagj + i * lagi]);
 	nKO -= (thresh <= ans[j * lagj + i * lagi]);
       }
@@ -224,9 +177,10 @@ void rschlatherdirect(double *coord, int *nObs, int *nSite, int *dim,
   PutRNGstate();
   //Lastly we multiply by 1 / E[max(0, Y)]
   const double imean = M_SQRT2 * M_SQRT_PI;
-  for (i=(neffSite * *nObs);i--;)    
+  for (int i = 0; i < (neffSite * *nObs); i++)    
     ans[i] *= imean;
-  
+
+  free(covmat); free(gp);
   return;
 }
 
